import greenfoot.*;

public class AClock extends World
{
    private Button actionButton; // the button to change to timer mode
    private Button displayButton; // the button to change to ditigal display
    private Hand hourHand; // for the hour hand
    private Hand minuteHand; // for the minute hand
    private Hand secondHand; // for the second hand
    private Hand subSecondHand; // for the sub-second hand
    private int offset; // number of milliseconds to adjust for time zone
    
    public AClock()
    {
        super(800, 600, 1);
        /** the settings */
        Greenfoot.setSpeed(65);
        offset = java.util.TimeZone.getDefault().getRawOffset();
        /** the background */
        GreenfootImage bg = getBackground();
        bg.setColor(new Color(192, 224, 240));;
        bg.fillRect(0, 0, 598, 600);
        // small dot image
        GreenfootImage mark = new GreenfootImage(6, 6);
        mark.fillOval(0, 0, 5, 5);
        // large dot image
        GreenfootImage mark2 = new GreenfootImage(9, 9);
        mark2.fillOval(0, 0, 8, 8);
        // small inner dot image
        GreenfootImage mark3 = new GreenfootImage(6, 6);
        mark3.setColor(Color.ORANGE.darker());
        mark3.fillOval(0, 0, 5, 5);
        // large inner dot image
        GreenfootImage mark4 = new GreenfootImage(5, 5);
        mark4.setColor(Color.ORANGE.darker());
        mark4.fillOval(0, 0, 4, 4);
        // background image applicator
        BgInk ink = new BgInk();
        // apply images to background
        for (int i=0; i<60; i++)
        {
            if (i%3 == 0)
            {
                if (i%6 == 0)
                {
                    // prep inner number image
                    ink.setImage(new GreenfootImage(""+((i/6+2)%10+1), 36, Color.ORANGE.darker(), new Color(0, 0, 0, 0)));
                    // apply the inner number
                    addObject(ink, 300, 300);
                    ink.setRotation(i*6+18);
                    ink.move(130);
                    ink.turn(90);
                    if (i >= 0 && i < 30) ink.turn(180);
                    ink.apply();
                    // prep inner large dot image
                    ink.setImage(mark3);
                }
                else ink.setImage(mark4);
                addObject(ink, 300, 300);
                ink.setRotation(i*6+18);
                ink.move(112);
                ink.apply();
            }
            if (i%5 == 0)
            {
                // prep number image
                ink.setImage(new GreenfootImage(""+((i/5+2)%12+1), 72, Color.BLACK, new Color(0, 0, 0, 0)));
                // apply the number
                addObject(ink, 300, 300);
                ink.setRotation(i*6);
                ink.move(260);
                ink.turn(90);
                if (i > 0 && i < 30) ink.turn(180);
                ink.apply();
                // prep large dot image
                ink.setImage(mark2);
            }
            else
                // prep small dot image
                ink.setImage(mark);
            // apply the dot
            addObject(ink, 300, 300);
            ink.setRotation(i*6);
            ink.move(225);
            ink.apply();
        }
        bg.setColor(Color.BLACK);
        bg.fillRect(599, 0, 2, 599);
        bg.setColor(new Color(240, 224, 192));
        bg.fillRect(601, 0, 199, 599);
        String title = "ANALOG\nCLOCK";
        GreenfootImage titleImg = new GreenfootImage(title, 36, Color.BLACK, new Color(0, 0, 0, 0));
        bg.drawImage(titleImg, 700-titleImg.getWidth()/2, 10);
        /** the buttons */
        addObject(actionButton = new Button("Timer"), 700, 120);
        addObject(displayButton = new Button("Digital"), 700, 160);
        /** the hands */
        addObject(hourHand = new Hand(240, 7, Color.BLACK, 12*60*60), 300, 300); // the hour hand
        addObject(minuteHand = new Hand(280, 5, Color.BLACK, 60*60), 300, 300); // the minute hand
        addObject(secondHand = new Hand(270, 3, new Color(224, 0, 0), 60), 300, 300); // the second hand
        addObject(subSecondHand = new Hand(150, 2, Color.YELLOW.darker(), 1), 300, 300); // the tenth hand
        /** initialize hands */
        act();
    }
    
    /** keeps hands synchronized to current time */
    public void act()
    {
        /** adjust hand positions */
        long time = System.currentTimeMillis()+offset;
        hourHand.set(time);
        minuteHand.set(time);
        secondHand.set(time);
        subSecondHand.set(time);
        /** check for button clicks */
        if (Greenfoot.mouseClicked(actionButton)) Greenfoot.setWorld(new ATimer());
        if (Greenfoot.mouseClicked(displayButton)) Greenfoot.setWorld(new DClock());
    }    
}