import greenfoot.*;

/** 
 * CLASS: BgInk (subclass of Actor)<br>
 * AUTHOR: danpost (greenfoot.org username)<br>
 * DATE: February 27, 2015<br>
 * <br>
 * DESCRIPTION:  a simple class to facilitate drawing images onto the background
 * of the world<br>
 * <br>
 * USE:<br>
 * (1) create an new instance of this class<br>
 * (2) give it the image you wish to draw on the world background<br>
 * (3) add the BgInk object into the world<br>
 * (4) any combination of moving and turning to position the image as needed<br>
 * (5) set the final rotation of the actor (turning the actor, in essence, turns its image)<br>
 * (6) call the 'apply' method on the object to paint its image on the background
 * of the world as currently positioned; the object will remove itself when its
 * image is applied;<br>
 * (7) repeat steps 2 through 6 to draw more images onto the world background<br>
 * <br>
 * For example: to drawing the numbers of the face of a clock on the world backgound
 * normally require a bunch of calculations to determine where the images are would to
 * be drawn (plus any rotation would need to be dealt with before drawing the image on
 * the background).  The following code shows how the numbers can be drawn easily:<br>
 * <br>
   <pre>   BgInk ink = new BgInk(); // step 1
   for (int i = 0; i &lt 12; i++) // step 7, for each number on the clock face
   {
       ink.setImage(new GreenfootImage(""+((i+2)%12+1), 72, null, null)); // step 2, the image of the number
       addObject(ink, 300, 300); // step 3, placing object at center of clock
       ink.setRotation(i*30); // step 4, point in direction of where number goes
       ink.move(260); // step 4 (continued), move to final location of image
       ink.turn(90); // step 5, set standard rotation of the actor
       if (i &gt 0 && i &lt 6) ink.turn(180); // step 5 (continued), flip actor for lower numbers on clock
       ink.apply(); // step 6
   }</pre>
 */
public class BgInk extends Actor
{
    /** draws its image onto world background with its rotation and removes self */
    public void apply()
    {
        int degrees = getRotation(); // the rotation of the BgInk object
        GreenfootImage img = getImage(); // non-rotated image
        GreenfootImage image = null; // for final image
        if (degrees != 0)
        {
            // use a rotated image
            int w = img.getWidth(), h = img.getHeight();
            int size = Math.max(w, h)*3/2;
            image = new GreenfootImage(size, size);
            image.drawImage(img, (size-w)/2, (size-h)/2);
            image.rotate(degrees);
        }
        else
        {
            // use the non-rotated image
            image = new GreenfootImage(img);
        }
        // draw image onto world background
        int imgX = getX()-image.getWidth()/2;
        int imgY = getY()-image.getHeight()/2;
        getWorld().getBackground().drawImage(image, imgX, imgY);
        // remove self
        getWorld().removeObject(this);
    }
}