import greenfoot.*;

public class D2Timer extends World
{
    private Button actionButton; // for the button to change to clock mode
    private Button displayButton; // for the button to change to analog display
    private Button soloButton; // for the button to change to using a solo timer
    private Button helpButton; // for the button that displays the help screen
    private Button startButton; // for the button that starts the timer
    private Button stopButton; // for the button that stops the timer
    private Button resetButton; // for the button that resets the timer
    private Panel[] hourPanel = new Panel[2]; // for the hour digits panel
    private Panel[] minutePanel = new Panel[2]; // for the minute digits panel
    private Panel[] secondPanel = new Panel[2]; // for the second digits panel
    private Panel[] subSecondPanel = new Panel[2]; // for the sub-second digits panel
    private long[] initTime = new long[2]; // for the initial starting times
    private long[] remainTime = new long[2]; // for the times remaining for the timers when last stopped
    private long beginTime; // for the time when current running timer was started
    private long[] adjustment = new long[2]; // for the increments per turn
    private int running = -1; // to point to running timer (value for timer; + for running; - for not running)
    
    public D2Timer()
    {
        super(800, 600, 1);
        /** the settings */
        Greenfoot.setSpeed(65);
        /** the background */
        GreenfootImage bg = getBackground();
        bg.setColor(new Color(240, 224, 240));
        bg.fillRect(0, 0, 598, 300);
        bg.setColor(new Color(160, 160, 128));
        bg.fillRect(0, 300, 598, 300);
        bg.setColor(Color.BLACK);
        bg.fillRect(0, 298, 600, 5);
        bg.fillOval(145, 135, 7, 7);
        bg.fillOval(145, 165, 7, 7);
        bg.fillOval(445, 180, 7, 7);
        bg.fillOval(145, 435, 7, 7);
        bg.fillOval(145, 465, 7, 7);
        bg.fillOval(445, 480, 7, 7);
        bg.fillRect(599, 0, 2, 599);
        bg.setColor(new Color(240, 224, 192));
        bg.fillRect(601, 0, 199, 599);
        String title = "DIGITAL\nDUAL TIMER";
        GreenfootImage titleImg = new GreenfootImage(title, 36, Color.BLACK, new Color(0, 0, 0, 0));
        bg.drawImage(titleImg, 700-titleImg.getWidth()/2, 10);
        /** the digit panels */
        addObject(hourPanel[0] = new Panel(12*60*60, 12, 0, Color.WHITE), 75, 150);
        addObject(minutePanel[0] = new Panel(60*60, 60, 2, Color.WHITE), 225, 150);
        addObject(secondPanel[0] = new Panel(60, 60, 2, Color.MAGENTA), 375, 150);
        addObject(subSecondPanel[0] = new Panel(1, 100, 2, Color.MAGENTA), 525, 150);
        addObject(hourPanel[1] = new Panel(12*60*60, 12, 0, Color.WHITE), 75, 450);
        addObject(minutePanel[1] = new Panel(60*60, 60, 2, Color.WHITE), 225, 450);
        addObject(secondPanel[1] = new Panel(60, 60, 2, Color.MAGENTA), 375, 450);
        addObject(subSecondPanel[1] = new Panel(1, 100, 2, Color.MAGENTA), 525, 450);
        /** the buttons */
        addObject(actionButton = new Button("Timer"), 700, 120);
        addObject(displayButton = new Button("Analog"), 700, 160);
        addObject(soloButton = new Button("Solo"), 700, 200);
        addObject(helpButton = new Button("Help"), 700, 450);
        addObject(stopButton = new Button("Stop"), 700, 490);
        addObject(resetButton = new Button("Reset"), 700, 530);
        addObject(startButton = new Button("Start"), 700, 570);
        addObject(new Help(), 400, 300);
    }
    
    /** starts the timer */
    private void start()
    {
        beginTime = System.currentTimeMillis();
        running = -running;
    }
    
    /** stops the timer */
    private void stop()
    {
        remainTime[running-1] += beginTime-System.currentTimeMillis();
        running = -running;
    }
    
    /** change which timer is running */
    private void change()
    {
        /** stopping current timer */
        remainTime[running-1] += beginTime-System.currentTimeMillis();
        if (remainTime[running-1] <= 0) remainTime[running-1] = 0;
        else remainTime[running-1] += adjustment[running-1];
        hourPanel[running-1].set(remainTime[running-1]);
        minutePanel[running-1].set(remainTime[running-1]);
        secondPanel[running-1].set(remainTime[running-1]);
        /** starting new timer */
        running = running%2+1;
        beginTime = System.currentTimeMillis();
    }
    
    /** updates panel displays and checks for mouse clicks */
    public void act()
    {
        if (running > 0)
        {
            /** update timer display */
            long time = remainTime[running-1]-(System.currentTimeMillis()-beginTime);
            if (time < 0) time = 0;
            hourPanel[running-1].set(time);
            minutePanel[running-1].set(time);
            secondPanel[running-1].set(time);
            subSecondPanel[running-1].set(time);
            /** check for request to change timers or stop timer */
            if (Greenfoot.mouseClicked(stopButton)) stop();
            else if (Greenfoot.mouseClicked(null)) change();
        }
        else
        {
            /** check for button clicks */
            if (!Greenfoot.mouseClicked(null)) return;
            if (Greenfoot.mouseClicked(actionButton)) { Greenfoot.setWorld(new DClock()); return; }
            if (Greenfoot.mouseClicked(displayButton)) { Greenfoot.setWorld(new ATimer()); return; }
            if (Greenfoot.mouseClicked(soloButton)) { Greenfoot.setWorld(new DTimer()); return; }
            if (Greenfoot.mouseClicked(helpButton)) { addObject(new Help(), 400, 300); return; }
            if (Greenfoot.mouseClicked(resetButton)) { Greenfoot.setWorld(new D2Timer()); return; }
            if (Greenfoot.mouseClicked(startButton)) { start(); return; }
            /** check for panel clicks */
            int button = Greenfoot.getMouseInfo().getButton();
            if (button != 1 && button != 3) return;
            int factor = 0;
            if (button == 1) factor = 10; else factor = 1;            
            for (int i=0; i<2; i++)
            {
                if (Greenfoot.mouseClicked(hourPanel[i]))
                {
                    initTime[i] += factor*60*60*1000;
                    hourPanel[i].set(initTime[i]);
                    break;
                }
                if (Greenfoot.mouseClicked(minutePanel[i]))
                {
                    initTime[i] += factor*60*1000;
                    minutePanel[i].set(initTime[i]);
                    break; 
                }
                if (Greenfoot.mouseClicked(secondPanel[i]))
                {
                    adjustment[i] += factor*1000;
                    secondPanel[i].set(adjustment[i]);
                    break; 
                }
            }
            remainTime[0] = initTime[0];
            remainTime[1] = initTime[1];
        }
    }    
}