import greenfoot.*;

public class DClock extends World
{
    private Button actionButton; // for the button to change to timer mode
    private Button displayButton; // for the button to change to analog display
    private Panel hourPanel; // for the hours digits panel
    private Panel minutePanel; // for the minutes digits panel
    private Panel secondPanel; // for the seconds digits panel
    private Panel subSecondPanel; // for the partial second digits panels
    private int offset; // number of milliseconds to adjust for time zone
    
    public DClock()
    {
        super(800, 600, 1);
        /** the settings */
        Greenfoot.setSpeed(65);
        offset = java.util.TimeZone.getDefault().getRawOffset();
        /** the background */
        GreenfootImage bg = getBackground();
        bg.setColor(new Color(192, 224, 240));;
        bg.fillRect(0, 0, 598, 600);
        bg.setColor(Color.BLACK);
        bg.fillOval(145, 285, 7, 7);
        bg.fillOval(145, 315, 7, 7);
        bg.fillOval(445, 330, 7, 7);
        bg.fillRect(599, 0, 2, 599);
        bg.setColor(new Color(240, 224, 192));
        bg.fillRect(601, 0, 199, 599);
        String title = "DIGITAL\nCLOCK";
        GreenfootImage titleImg = new GreenfootImage(title, 36, Color.BLACK, new Color(0, 0, 0, 0));
        bg.drawImage(titleImg, 700-titleImg.getWidth()/2, 10);
        /** the digit panels */
        addObject(hourPanel = new Panel(12*60*60, 12, 0, Color.WHITE), 75, 300);
        addObject(minutePanel = new Panel(60*60, 60, 2, Color.WHITE), 225, 300);
        addObject(secondPanel = new Panel(60, 60, 2, Color.MAGENTA), 375, 300);
        addObject(subSecondPanel = new Panel(1, 100, 2, Color.MAGENTA), 525, 300);
        /** the buttons */
        addObject(actionButton = new Button("Timer"), 700, 120);
        addObject(displayButton = new Button("Analog"), 700, 160);
        /** initialize digit panels */
        act();
    }
    
    /** keeps hands synchronized to current time */
    public void act()
    {
        /** update panel digits */
        long time = System.currentTimeMillis()+offset;
        hourPanel.set(time);
        minutePanel.set(time);
        secondPanel.set(time);
        subSecondPanel.set(time);
        boolean build = false;
        /** check for button clicks */
        if (Greenfoot.mouseClicked(actionButton)) Greenfoot.setWorld(new DTimer());
        if (Greenfoot.mouseClicked(displayButton)) Greenfoot.setWorld(new AClock());
    }    
}