import greenfoot.*;

public class DTimer extends World
{
    private Button actionButton; // for the button to change to clock mode
    private Button displayButton; // for the button to change to analog display
    private Button dualButton; // for the button to change to using dual timers
    private Button startButton; // for the button that starts the timer
    private Button resetButton; // for the button that resets the timer
    private Panel hourPanel; // for the hour digits panel
    private Panel minutePanel; // for the minute digits panel
    private Panel secondPanel; // for the second digits panel
    private Panel subSecondPanel; // for the sub-second digits panel
    private long elapsedTime; // for the total running time of the timer
    private long offset; // for the adjustment from real time
    private boolean running; // for the running state of the timer
    
    public DTimer()
    {
        super(800, 600, 1);
        /** the settings */
        Greenfoot.setSpeed(65);
        /** the background */
        GreenfootImage bg = getBackground();
        bg.setColor(new Color(192, 224, 240));;
        bg.fillRect(0, 0, 598, 600);
        bg.setColor(Color.BLACK);
        bg.fillOval(145, 285, 7, 7);
        bg.fillOval(145, 315, 7, 7);
        bg.fillOval(445, 330, 7, 7);
        bg.fillRect(599, 0, 2, 599);
        bg.setColor(new Color(240, 224, 192));
        bg.fillRect(601, 0, 199, 599);
        String title = "DIGITAL\nTIMER";
        GreenfootImage titleImg = new GreenfootImage(title, 36, Color.BLACK, new Color(0, 0, 0, 0));
        bg.drawImage(titleImg, 700-titleImg.getWidth()/2, 10);
        /** the digit panels */
        addObject(hourPanel = new Panel(12*60*60, 12, 0, Color.WHITE), 75, 300);
        addObject(minutePanel = new Panel(60*60, 60, 2, Color.WHITE), 225, 300);
        addObject(secondPanel = new Panel(60, 60, 2, Color.MAGENTA), 375, 300);
        addObject(subSecondPanel = new Panel(1, 100, 2, Color.MAGENTA), 525, 300);
        /** the buttons */
        addObject(actionButton = new Button("Timer"), 700, 120);
        addObject(displayButton = new Button("Analog"), 700, 160);
        addObject(dualButton = new Button("Dual"), 700, 200);
        addObject(resetButton = new Button("Reset"), 700, 530);
        addObject(startButton = new Button("Start"), 700, 570);
        /** initialize the timer */
        start();
        act();
        stop();
    }
    
    /** starts the timer */
    private void start()
    {
        offset = -System.currentTimeMillis()+elapsedTime;
        running = true;
    }
    
    /** stops the timer */
    private void stop()
    {
        elapsedTime = System.currentTimeMillis()+offset;
        running = false;
    }
    
    /** updates hand positions and checks for button clicks */
    public void act()
    {
        if (running)
        {
            /** update timer display */
            long time = System.currentTimeMillis()+offset;
            hourPanel.set(time);
            minutePanel.set(time);
            secondPanel.set(time);
            subSecondPanel.set(time);
            /** check for request to stop timer */
            if (Greenfoot.mouseClicked(null)) stop();
        }
        else
        {
            /** check for button clicks */
            if (Greenfoot.mouseClicked(actionButton)) Greenfoot.setWorld(new DClock());
            if (Greenfoot.mouseClicked(displayButton)) Greenfoot.setWorld(new ATimer());
            if (Greenfoot.mouseClicked(dualButton)) Greenfoot.setWorld(new D2Timer());
            if (Greenfoot.mouseClicked(resetButton)) Greenfoot.setWorld(new DTimer());
            if (Greenfoot.mouseClicked(startButton)) start();
        }
    }    
}